@@ -293,7 +293,7 @@ repositories = utils.InsensitivePreservingDict({
                                                    'SupyPlugins',
                                                    ),
                'Iota':             GithubRepository(
-                                                   'ZeeCrazyAtheist',
+                                                   'IotaSpencer',
                                                    'supyplugins',
                                                    ),
                'waratte':          GithubRepository(
@@ -308,6 +308,11 @@ repositories = utils.InsensitivePreservingDict({
                                                    'prgmrbill',
                                                    'limnoria-plugins',
                                                    ),
+                'fudster':         GithubRepository(
+                                                   'fudster',
+                                                   'supybot-plugins',
+                                                   ),
+
                })
 
 class PluginDownloader(callbacks.Plugin):
