@@ -156,8 +156,8 @@ if os.name == 'posix':
             self.assertError('unix ping6 --W a --c 1 ::1')
 
         def testCall(self):
-            self.assertNotError('unix call /bin/ls .')
-            self.assertRegexp('unix call /bin/ls .', 'src,')
+            self.assertNotError('unix call /bin/ls /')
+            self.assertRegexp('unix call /bin/ls /', 'boot, .*dev, ')
             self.assertError('unix call /usr/bin/nosuchcommandaoeuaoeu')
 
         def testUptime(self):
