@@ -378,21 +378,43 @@ class IrcStateTestCase(SupyTestCase):
 class IrcTestCase(SupyTestCase):
     def setUp(self):
         self.irc = irclib.Irc('test')
+
         #m = self.irc.takeMsg()
         #self.failUnless(m.command == 'PASS', 'Expected PASS, got %r.' % m)
+
+        m = self.irc.takeMsg()
+        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
+        self.failUnless(m.args == ('LS', '302'), 'Expected CAP LS 302, got %r.' % m)
+
         m = self.irc.takeMsg()
         self.failUnless(m.command == 'NICK', 'Expected NICK, got %r.' % m)
-        m = self.irc.takeMsg()
-        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
-        m = self.irc.takeMsg()
-        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
-        m = self.irc.takeMsg()
-        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
-        m = self.irc.takeMsg()
-        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
+
         m = self.irc.takeMsg()
         self.failUnless(m.command == 'USER', 'Expected USER, got %r.' % m)
 
+        # TODO
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+            args=('*', 'LS', '*', 'account-tag multi-prefix')))
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+            args=('*', 'LS', 'extended-join')))
+
+        m = self.irc.takeMsg()
+        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
+        self.assertEqual(m.args[0], 'REQ', m)
+        # NOTE: Capabilities are requested in alphabetic order, because
+        # sets are unordered, and their "order" is nondeterministic.
+        self.assertEqual(m.args[1], 'account-tag extended-join multi-prefix')
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+            args=('*', 'ACK', 'account-tag multi-prefix extended-join')))
+
+        m = self.irc.takeMsg()
+        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
+        self.assertEqual(m.args, ('END',), m)
+
+        m = self.irc.takeMsg()
+        self.failUnless(m is None, m)
+
     def testPingResponse(self):
         self.irc.feedMsg(ircmsgs.ping('123'))
         self.assertEqual(ircmsgs.pong('123'), self.irc.takeMsg())
@@ -431,7 +453,6 @@ class IrcTestCase(SupyTestCase):
             except:
                 pass
         self.irc.reset()
-        self.failIf(self.irc.fastqueue)
         self.failIf(self.irc.state.history)
         self.failIf(self.irc.state.channels)
         self.failIf(self.irc.outstandingPing)
@@ -445,6 +466,252 @@ class IrcTestCase(SupyTestCase):
         self.irc.feedMsg(msg2)
         self.assertEqual(list(self.irc.state.history), [msg1, msg2])
 
+    def testQuit(self):
+        self.irc.reset()
+        self.irc.feedMsg(ircmsgs.IrcMsg(':someuser JOIN #foo'))
+        self.irc.feedMsg(ircmsgs.IrcMsg(':someuser JOIN #bar'))
+        self.irc.feedMsg(ircmsgs.IrcMsg(':someuser2 JOIN #bar2'))
+        class Callback(irclib.IrcCallback):
+            channels_set = None
+            def name(self):
+                return 'testcallback'
+            def doQuit(self2, irc, msg):
+                self2.channels_set = msg.tagged('channels')
+        c = Callback()
+        self.irc.addCallback(c)
+        try:
+            self.irc.feedMsg(ircmsgs.IrcMsg(':someuser QUIT'))
+        finally:
+            self.irc.removeCallback(c.name())
+        self.assertEqual(c.channels_set, ircutils.IrcSet(['#foo', '#bar']))
+
+    def testNick(self):
+        self.irc.reset()
+        self.irc.feedMsg(ircmsgs.IrcMsg(':someuser JOIN #foo'))
+        self.irc.feedMsg(ircmsgs.IrcMsg(':someuser JOIN #bar'))
+        self.irc.feedMsg(ircmsgs.IrcMsg(':someuser2 JOIN #bar2'))
+        class Callback(irclib.IrcCallback):
+            channels_set = None
+            def name(self):
+                return 'testcallback'
+            def doNick(self2, irc, msg):
+                self2.channels_set = msg.tagged('channels')
+        c = Callback()
+        self.irc.addCallback(c)
+        try:
+            self.irc.feedMsg(ircmsgs.IrcMsg(':someuser NICK newuser'))
+        finally:
+            self.irc.removeCallback(c.name())
+        self.assertEqual(c.channels_set, ircutils.IrcSet(['#foo', '#bar']))
+
+    def testBatch(self):
+        self.irc.reset()
+        self.irc.feedMsg(ircmsgs.IrcMsg(':someuser1 JOIN #foo'))
+        self.irc.feedMsg(ircmsgs.IrcMsg(':host BATCH +name netjoin'))
+        m1 = ircmsgs.IrcMsg('@batch=name :someuser2 JOIN #foo')
+        self.irc.feedMsg(m1)
+        self.irc.feedMsg(ircmsgs.IrcMsg(':someuser3 JOIN #foo'))
+        m2 = ircmsgs.IrcMsg('@batch=name :someuser4 JOIN #foo')
+        self.irc.feedMsg(m2)
+        class Callback(irclib.IrcCallback):
+            batch = None
+            def name(self):
+                return 'testcallback'
+            def doBatch(self2, irc, msg):
+                self2.batch = msg.tagged('batch')
+        c = Callback()
+        self.irc.addCallback(c)
+        try:
+            self.irc.feedMsg(ircmsgs.IrcMsg(':host BATCH -name'))
+        finally:
+            self.irc.removeCallback(c.name())
+        self.assertEqual(c.batch, irclib.Batch('netjoin', (), [m1, m2]))
+
+class SaslTestCase(SupyTestCase):
+    def setUp(self):
+        pass
+
+    def startCapNegociation(self, caps='sasl'):
+        m = self.irc.takeMsg()
+        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
+        self.failUnless(m.args == ('LS', '302'), 'Expected CAP LS 302, got %r.' % m)
+
+        m = self.irc.takeMsg()
+        self.failUnless(m.command == 'NICK', 'Expected NICK, got %r.' % m)
+
+        m = self.irc.takeMsg()
+        self.failUnless(m.command == 'USER', 'Expected USER, got %r.' % m)
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+            args=('*', 'LS', caps)))
+
+        if caps:
+            m = self.irc.takeMsg()
+            self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
+            self.assertEqual(m.args[0], 'REQ', m)
+            self.assertEqual(m.args[1], 'sasl')
+
+            self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+                args=('*', 'ACK', 'sasl')))
+
+    def endCapNegociation(self):
+        m = self.irc.takeMsg()
+        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
+        self.assertEqual(m.args, ('END',), m)
+
+    def testPlain(self):
+        try:
+            conf.supybot.networks.test.sasl.username.setValue('jilles')
+            conf.supybot.networks.test.sasl.password.setValue('sesame')
+            self.irc = irclib.Irc('test')
+        finally:
+            conf.supybot.networks.test.sasl.username.setValue('')
+            conf.supybot.networks.test.sasl.password.setValue('')
+        self.assertEqual(self.irc.sasl_current_mechanism, None)
+        self.assertEqual(self.irc.sasl_next_mechanisms, ['plain'])
+
+        self.startCapNegociation()
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('PLAIN',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='AUTHENTICATE', args=('+',)))
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('amlsbGVzAGppbGxlcwBzZXNhbWU=',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='900', args=('jilles',)))
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='903', args=('jilles',)))
+
+        self.endCapNegociation()
+
+    def testExternalFallbackToPlain(self):
+        try:
+            conf.supybot.networks.test.sasl.username.setValue('jilles')
+            conf.supybot.networks.test.sasl.password.setValue('sesame')
+            conf.supybot.networks.test.certfile.setValue('foo')
+            self.irc = irclib.Irc('test')
+        finally:
+            conf.supybot.networks.test.sasl.username.setValue('')
+            conf.supybot.networks.test.sasl.password.setValue('')
+            conf.supybot.networks.test.certfile.setValue('')
+        self.assertEqual(self.irc.sasl_current_mechanism, None)
+        self.assertEqual(self.irc.sasl_next_mechanisms,
+                ['external', 'plain'])
+
+        self.startCapNegociation()
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('EXTERNAL',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='904',
+            args=('mechanism not available',)))
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('PLAIN',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='AUTHENTICATE', args=('+',)))
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('amlsbGVzAGppbGxlcwBzZXNhbWU=',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='900', args=('jilles',)))
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='903', args=('jilles',)))
+
+        self.endCapNegociation()
+
+    def testFilter(self):
+        try:
+            conf.supybot.networks.test.sasl.username.setValue('jilles')
+            conf.supybot.networks.test.sasl.password.setValue('sesame')
+            conf.supybot.networks.test.certfile.setValue('foo')
+            self.irc = irclib.Irc('test')
+        finally:
+            conf.supybot.networks.test.sasl.username.setValue('')
+            conf.supybot.networks.test.sasl.password.setValue('')
+            conf.supybot.networks.test.certfile.setValue('')
+        self.assertEqual(self.irc.sasl_current_mechanism, None)
+        self.assertEqual(self.irc.sasl_next_mechanisms,
+                ['external', 'plain'])
+
+        self.startCapNegociation(caps='sasl=foo,plain,bar')
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('PLAIN',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='AUTHENTICATE', args=('+',)))
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('amlsbGVzAGppbGxlcwBzZXNhbWU=',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='900', args=('jilles',)))
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='903', args=('jilles',)))
+
+        self.endCapNegociation()
+
+    def testReauthenticate(self):
+        try:
+            conf.supybot.networks.test.sasl.username.setValue('jilles')
+            conf.supybot.networks.test.sasl.password.setValue('sesame')
+            self.irc = irclib.Irc('test')
+        finally:
+            conf.supybot.networks.test.sasl.username.setValue('')
+            conf.supybot.networks.test.sasl.password.setValue('')
+        self.assertEqual(self.irc.sasl_current_mechanism, None)
+        self.assertEqual(self.irc.sasl_next_mechanisms, ['plain'])
+
+        self.startCapNegociation(caps='')
+
+        self.endCapNegociation()
+
+        while self.irc.takeMsg():
+            pass
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+                args=('*', 'NEW', 'sasl=EXTERNAL')))
+
+        self.irc.takeMsg() # None. But even if it was CAP REQ sasl, it would be ok
+        self.assertEqual(self.irc.takeMsg(), None)
+
+        try:
+            conf.supybot.networks.test.sasl.username.setValue('jilles')
+            conf.supybot.networks.test.sasl.password.setValue('sesame')
+            self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+                    args=('*', 'DEL', 'sasl')))
+            self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+                    args=('*', 'NEW', 'sasl=PLAIN')))
+        finally:
+            conf.supybot.networks.test.sasl.username.setValue('')
+            conf.supybot.networks.test.sasl.password.setValue('')
+
+        m = self.irc.takeMsg()
+        self.failUnless(m.command == 'CAP', 'Expected CAP, got %r.' % m)
+        self.assertEqual(m.args[0], 'REQ', m)
+        self.assertEqual(m.args[1], 'sasl')
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='CAP',
+            args=('*', 'ACK', 'sasl')))
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('PLAIN',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='AUTHENTICATE', args=('+',)))
+
+        m = self.irc.takeMsg()
+        self.assertEqual(m, ircmsgs.IrcMsg(command='AUTHENTICATE',
+            args=('amlsbGVzAGppbGxlcwBzZXNhbWU=',)))
+
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='900', args=('jilles',)))
+        self.irc.feedMsg(ircmsgs.IrcMsg(command='903', args=('jilles',)))
+
+
 
 class IrcCallbackTestCase(SupyTestCase):
     class FakeIrc:
@@ -488,16 +755,13 @@ class IrcCallbackTestCase(SupyTestCase):
             user = 'user any user'
             conf.supybot.user.setValue(user)
             expected = [
+                ircmsgs.IrcMsg(command='CAP', args=('LS', '302')),
                 ircmsgs.nick(nick),
-                ircmsgs.IrcMsg(command='CAP', args=('REQ', 'account-notify')),
-                ircmsgs.IrcMsg(command='CAP', args=('REQ', 'extended-join')),
-                ircmsgs.IrcMsg(command='CAP', args=('REQ', 'multi-prefix')),
-                ircmsgs.IrcMsg(command='CAP', args=('END',)),
-                ircmsgs.user('limnoria', user)
+                ircmsgs.user('limnoria', user),
             ]
             irc = irclib.Irc('test')
             msgs = [irc.takeMsg()]
-            while msgs[-1] != None:
+            while msgs[-1] is not None:
                 msgs.append(irc.takeMsg())
             msgs.pop()
             self.assertEqual(msgs, expected)
@@ -505,10 +769,10 @@ class IrcCallbackTestCase(SupyTestCase):
             conf.supybot.networks.test.password.setValue(password)
             irc = irclib.Irc('test')
             msgs = [irc.takeMsg()]
-            while msgs[-1] != None:
+            while msgs[-1] is not None:
                 msgs.append(irc.takeMsg())
             msgs.pop()
-            expected.insert(0, ircmsgs.password(password))
+            expected.insert(1, ircmsgs.password(password))
             self.assertEqual(msgs, expected)
         finally:
             conf.supybot.nick.setValue(originalNick)
